# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2020 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
CORE POS (API) -> Rattail data importing
"""

import decimal
import logging

from corepos.api import CoreWebAPI
from corepos.db.office_op import Session as CoreSession, model as corepos

from rattail import importing
from rattail.gpc import GPC
from rattail.util import OrderedDict
from rattail.db.util import normalize_full_name, short_session
from rattail_corepos import importing as corepos_importing


log = logging.getLogger(__name__)


class FromCOREPOSToRattail(importing.ToRattailHandler):
    """
    Import handler for data coming from a CORE POS API.
    """
    host_title = "CORE-POS (API)"

    def get_importers(self):
        importers = OrderedDict()
        importers['Customer'] = CustomerImporter
        importers['Department'] = DepartmentImporter
        importers['Subdepartment'] = SubdepartmentImporter
        importers['Vendor'] = VendorImporter
        importers['Product'] = ProductImporter
        return importers


class FromCOREPOSAPI(importing.Importer):
    """
    Base class for all CORE POS API data importers.
    """

    def setup(self):
        super(FromCOREPOSAPI, self).setup()
        self.establish_api()

    def establish_api(self):
        url = self.config.require('corepos.api', 'url')
        self.api = CoreWebAPI(url)


class CustomerImporter(FromCOREPOSAPI, importing.model.CustomerImporter):
    """
    Importer for customer data from CORE POS API.
    """
    key = 'number'
    supported_fields = [
        'id',
        'number',
        'name',
        'first_name',
        'last_name',
    ]

    def get_host_objects(self):
        # TODO: ideally could do this, but API doesn't let us fetch "all"
        # return self.api.get_customers()

        # first we fetch all customer records from CORE DB
        with short_session(Session=CoreSession) as s:
            db_customers = s.query(corepos.Customer).all()
            s.expunge_all()

        # now we must fetch each customer account individually from API
        customers = OrderedDict()

        def fetch(dbcust, i):
            if dbcust.card_number in customers:
                return          # already fetched this one
            customer = self.api.get_customer(dbcust.card_number)
            if customer:
                customers[dbcust.card_number] = customer
            else:
                logger = log.warning if dbcust.account_holder else log.debug
                logger("could not fetch customer from CORE API: %s",
                       dbcust.card_number)

        self.progress_loop(fetch, db_customers,
                           message="Fetching Customer data from CORE-POS API")
        return list(customers.values())

    def normalize_host_object(self, customer):

        # figure out the "account holder" person for the customer
        people = customer['customers']
        account_holders = [person for person in people
                           if person['accountHolder']]
        if len(account_holders) > 1:
            log.warning("customer %s has %s account holders in CORE: %s",
                        customer['cardNo'], len(account_holders), customer)
        elif not account_holders:
            raise NotImplementedError("TODO: how to handle customer with no account holders?")
        person = account_holders[0]

        return {
            'id': customer['customerAccountID'],
            'number': customer['cardNo'],
            'name': normalize_full_name(person['firstName'],
                                        person['lastName']),
            'first_name': person['firstName'],
            'last_name': person['lastName'],
        }


class DepartmentImporter(FromCOREPOSAPI, importing.model.DepartmentImporter):
    """
    Importer for department data from CORE POS API.
    """
    key = 'number'
    supported_fields = [
        'number',
        'name',
    ]

    def get_host_objects(self):
        return self.api.get_departments()

    def normalize_host_object(self, department):
        return {
            'number': int(department['dept_no']),
            'name': department['dept_name'],
        }


class SubdepartmentImporter(FromCOREPOSAPI, importing.model.SubdepartmentImporter):
    """
    Importer for subdepartment data from CORE POS API.
    """
    key = 'number'
    supported_fields = [
        'number',
        'name',
        'department_number',
    ]

    def get_host_objects(self):
        return self.api.get_subdepartments()

    def normalize_host_object(self, subdepartment):
        department_number = None
        if 'dept_ID' in subdepartment:
            department_number = int(subdepartment['dept_ID'])

        return {
            'number': int(subdepartment['subdept_no']),
            'name': subdepartment['subdept_name'],
            'department_number': department_number,
        }


class VendorImporter(FromCOREPOSAPI, corepos_importing.model.VendorImporter):
    """
    Importer for vendor data from CORE POS API.
    """
    key = 'corepos_id'
    supported_fields = [
        'corepos_id',
        'name',
        'abbreviation',
        'special_discount',
        'phone_number',
        'fax_number',
        'email_address',
    ]

    def get_host_objects(self):
        return self.api.get_vendors()

    def normalize_host_object(self, vendor):
        return {
            'corepos_id': int(vendor['vendorID']),
            'name': vendor['vendorName'],
            'abbreviation': vendor['vendorAbbreviation'] or None,
            'special_discount': decimal.Decimal(vendor['discountRate']),
            'phone_number': vendor.get('phone') or None,
            'fax_number': vendor.get('fax') or None,
            'email_address': vendor.get('email') or None,
        }


class ProductImporter(FromCOREPOSAPI, importing.model.ProductImporter):
    """
    Importer for product data from CORE POS API.
    """
    key = 'item_id'
    supported_fields = [
        'item_id',
        'upc',
        'brand_name',
        'description',
        'size',
        'weighed',
        'department_number',
        'subdepartment_number',
        'regular_price_price',
        'regular_price_multiple',
        'regular_price_type',
        'food_stampable',
        # 'tax1',
        # 'tax2',
    ]

    def get_host_objects(self):
        return self.api.get_products()

    def normalize_host_object(self, product):
        try:
            upc = GPC(product['upc'], calc_check_digit='upc')
        except (TypeError, ValueError):
            log.debug("CORE POS product has invalid UPC: %s", product['upc'])
            if len(self.key) == 1 and self.key[0] == 'upc':
                return
            upc = None

        department_number = None
        if 'department' in product:
            department_number = int(product['department'])

        subdepartment_number = None
        if 'subdept' in product:
            subdepartment_number = int(product['subdept']) or None

        price = None
        if product['normal_price'] is not None:
            price = decimal.Decimal(product['normal_price'])

        return {
            'item_id': product['upc'],
            'upc': upc,
            'brand_name': product.get('brand') or None,
            'description': product.get('description') or '',
            'size': product.get('size', '').strip() or None,

            'department_number': department_number,
            'subdepartment_number': subdepartment_number,

            'weighed': product['scale'] == '1',
            'food_stampable': product['foodstamp'] == '1',
            # 'tax1': product['tax'] == '1', # TODO: is this right?
            # 'tax2': product['tax'] == '2', # TODO: is this right?

            'regular_price_price': price,
            'regular_price_multiple': 1 if price is not None else None,
            'regular_price_type': self.enum.PRICE_TYPE_REGULAR if price is not None else None,
        }
